<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://competition.demo.getfoundni.co.uk/
 * @since      1.0.0
 *
 * @package    Gfni_Spot_The_Ball_Addon
 * @subpackage Gfni_Spot_The_Ball_Addon/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Gfni_Spot_The_Ball_Addon
 * @subpackage Gfni_Spot_The_Ball_Addon/admin
 * @author     Get Found NI <ryan@getfoundni.co.uk>
 */
class Gfni_Spot_The_Ball_Addon_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() { 

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/gfni-spot-the-ball-addon-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() { 

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/gfni-spot-the-ball-addon-admin.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'alpine', 'https://cdnjs.cloudflare.com/ajax/libs/alpinejs/2.3.0/alpine.js', array(), '2.3.0', true );

	}

	public function add_spot_the_ball_option()
	{
		if( function_exists('acf_add_local_field_group') ):

			acf_add_local_field_group(array(
				'key' => 'group_60ff403196b43',
				'title' => 'Spot The Ball',
				'fields' => array(
					array(
						'key' => 'field_60ff403ebd924',
						'label' => 'Enable spot the ball',
						'name' => 'enable_spot_the_ball',
						'type' => 'true_false',
						'instructions' => '',
						'required' => 0,
						'conditional_logic' => 0,
						'wrapper' => array(
							'width' => '',
							'class' => '',
							'id' => '',
						),
						'message' => '',
						'default_value' => 0,
						'ui' => 0,
						'ui_on_text' => '',
						'ui_off_text' => '',
					),
					array(
						'key' => 'field_60ff405ebd925',
						'label' => 'Spot the ball image',
						'name' => 'spot_the_ball_image',
						'type' => 'image',
						'instructions' => 'Add a high quality image to use for spot the ball style quiz',
						'required' => 0,
						'conditional_logic' => array(
							array(
								array(
									'field' => 'field_60ff403ebd924',
									'operator' => '==',
									'value' => '1',
								),
							),
						),
						'wrapper' => array(
							'width' => '',
							'class' => '',
							'id' => '',
						),
						'return_format' => 'array',
						'preview_size' => 'thumbnail',
						'library' => 'all',
						'min_width' => '',
						'min_height' => '',
						'min_size' => '',
						'max_width' => '',
						'max_height' => '',
						'max_size' => '',
						'mime_types' => '',
					),
				),
				'location' => array(
					array(
						array(
							'param' => 'post_type',
							'operator' => '==',
							'value' => 'product',
						),
					),
				),
				'menu_order' => 0,
				'position' => 'normal',
				'style' => 'default',
				'label_placement' => 'top',
				'instruction_placement' => 'label',
				'hide_on_screen' => '',
				'active' => true,
				'description' => '',
			));
			
		endif;
	}  

	function woocommerce_simple_lottery_meta() {

		global $post;

		$product_data = wc_get_product( $post->ID );
		if ( $product_data ) {
			$product_data_type = method_exists( $product_data, 'get_type' ) ? $product_data->get_type() : $product_data->product_type;
			if ( $product_data_type == 'lottery' ) {
				if( function_exists('get_field') && get_field('enable_spot_the_ball') ):
					add_meta_box( 'spot-the-ball-show', 'Spot the ball Image', array( $this, 'spot_the_ball_meta_callback' ), 'product', 'normal', 'default' );
				endif;
			}
		}

	}


	public function spot_the_ball_meta_callback()
	{
		global $product;
		global $post;
		$colspan = 9;
		$product_data    = wc_get_product( $post->ID ); 
		$use_answers 	 = wc_lottery_use_answers( $post->ID );
		$use_ticket_numbers = get_post_meta( $post->ID , '_lottery_use_pick_numbers', true );
		$answers = maybe_unserialize( get_post_meta( $post->ID, '_lottery_pn_answers', true ) );
		
		$lottery_history = apply_filters( 'woocommerce__lottery_history_data', $product_data->lottery_history() ); 

		$axis = [];
		$orderids = [];

		if ( $lottery_history ) : 

			foreach($lottery_history as $history_value){
				if(!in_array( $history_value->order_id , $orderids)){
					$orderids[] = $history_value->order_id;
				}
			}

			foreach( $orderids as $id ) { 

				$order = wc_get_order( $id); 

				foreach( $order->get_items() as $item ){    

					foreach( $item->get_formatted_meta_data( '_', true ) as $meta){
						if($meta->key == 'Guess'){
							$ax = explode(' ',$meta->value);
							$cord = [
								'x' => $ax[1],
								'y' => $ax[3],
							];
						}
						if($meta->key == 'Ticket number' && (isset($cord) && is_array($cord)) ){
							$axis[] = array_merge($cord, ['ticket' => $meta->value]);
							$cord = null;
						}
					} 
				}
			}
		endif;

		//echo '<pre>' . print_r(array_filter($axis),true) . '</pre>';
		//echo '<pre>' . print_r($orderids,true) . '</pre>';
		//exit;die; 

		ob_start(); ?> 
		<div class="spot-the-ball" x-data="guesses()" x-init="checkIfDrawn">

			<!-- Trigger/Open The Modal -->
			<h4 x-show="results.length == 0">Open the game below to set the winner or set winning X&Y coordinates manually below. Update the page to save.</h4>
			<span class="single_add_to_cart_button button alt" @click="modal = true">Open Game</span>

			<div>
				<div class="pick-winner">
					<label for="x">Winning X Axis: 
					<input type="text" x-model="x" name="cordx"></label>
					<label for="y">Winning Y Axis: 
					<input type="text" x-model="y" name="cordy"></label>
					<button @click.prevent="calculateWinner()">Show Results</button>
				</div>

				<div id="spotlist" class="spotlist" x-show="results.length > 0">  
					<button class="button button-primary button-large" @click.prevent="downloadResults()">Download Results</button>
					
					<h4>Winning Coordinates: X = <span x-text="x"></span>, Y = <span x-text="y"></span></h4>
					
					<table id="results" class="zui-table" cellspacing="0">
						<thead>
							<tr>
								<th>Place</th>
								<th>Ticket Number</th> 
								<th>Distance Away</th>
								<th>X Axis</th>
								<th>Y Axis</th>
							</tr>
						</thead>
						<tbody>
							<tr>
								<td>Winning Coordinates:</td>
								<td>-</td> 
								<td>-</td>
								<td x-text="x">X Axis</td>
								<td x-text="y">Y Axis</td>
							</tr>
							<template x-for="(cus,index) in results">	
								<tr>
									<td x-text="index + 1"></td>
									<td x-text="cus.ticket"></td>
									<td x-text="cus.distance"></td>
									<td x-text="cus.x"></td>
									<td x-text="cus.y"></td>
								</tr>						 
							</template> 
						</tbody>

					</table>

				</div> 
			</div>

			<!-- The Modal -->
			<div id="myModal" class="modal" x-show="modal" x-cloak>

				<!-- Modal content -->
				<div class="modal-content" @click.away="modal = false" style="width: <?php echo get_field('spot_the_ball_image')["width"]; ?>px;">
					<img id="hotspot_image" src="<?php echo get_field('spot_the_ball_image')["url"]; ?>" @click="setWin($event)" 
							width="<?php echo get_field('spot_the_ball_image')["width"]; ?>px" 
							height="<?php echo get_field('spot_the_ball_image')["height"]; ?>px">  

					<template x-for="(guess,index) in customers">
						<img class="cross" src="<?php echo plugin_dir_url( __FILE__ ).'images/cross.png';?>" :style="`left: ${(guess.x - 5)}px;top: ${(guess.y - 5)}px;`">
					</template>


					<h4 x-show="results.length == 0">Click on Image to set winning X&Y coordinates or enter manually below.</h4>
					<div class="pick-winner">
						<label for="x">Winning X Axis: 
						<input type="text" x-model="x" name="cordx"></label>
						<label for="y">Winning Y Axis: 
						<input type="text" x-model="y" name="cordy"></label>
						<button @click.prevent="calculateWinner()">Show Results</button>
					</div>

					<div id="spotlist" class="spotlist" x-show="results.length > 0">  
						<button class="button button-primary button-large" @click.prevent="downloadResults()">Download Results</button>
						
						<h4>Winning Coordinates: X = <span x-text="x"></span>, Y = <span x-text="y"></span></h4>
						
						<table id="results" class="zui-table" cellspacing="0">
							<thead>
								<tr>
									<th>Place</th>
									<th>Ticket Number</th> 
									<th>Distance Away</th>
									<th>X Axis</th>
									<th>Y Axis</th>
								</tr>
							</thead>
							<tbody>
								<tr>
									<td>Winning Coordinates:</td>
									<td>-</td> 
									<td>-</td>
									<td x-text="x">X Axis</td>
									<td x-text="y">Y Axis</td>
								</tr>
								<template x-for="(cus,index) in results">	
									<tr>
										<td x-text="index + 1"></td>
										<td x-text="cus.ticket"></td>
										<td x-text="cus.distance"></td>
										<td x-text="cus.x"></td>
										<td x-text="cus.y"></td>
									</tr>						 
								</template> 
							</tbody>

						</table>

					</div>
					<button class="btn" @click.prevent="modal = false">Exit</button>
				</div>

			</div> 
  

			 
			<script>
				function guesses() {
					return { 
						guesses: [],
						x: <?php echo (int)get_post_meta( $post->ID, '_winning_cord_x', true ) ?? 0 ?>,
						y: <?php echo (int)get_post_meta( $post->ID, '_winning_cord_y', true ) ?? 0 ?>,
						customers: <?php echo json_encode($axis); ?>,
						modal: false,  
						results: [],
						setWin(evt) { 
							const pos = evt.currentTarget.getBoundingClientRect(); 
							this.x = Math.round(evt.clientX - pos.left);
							this.y = Math.round(evt.clientY - pos.top); 
							this.calculateWinner();
						},
						checkIfDrawn(){
							if(this.x > 0 && this.y > 0 ){
								this.calculateWinner();
							}
						},
						formatCus(c){ 
							return 'Distance ' + c.distance +', Ticket: '+ c.ticket + ', x:' + c.x +', y:' + c.y;
						},
						formatGuess(g){
							return 'x:' + g.x +', y:' + g.y;
						},
						removeGuess(id) { 
							this.customers.splice(id,1); 
						},
						getDistance(x1, y1, x2, y2){
							let y = x2 - x1;
							let x = y2 - y1;
							
							return Math.sqrt(x * x + y * y);
						},
						calculateWinner(){

							this.results = [];
							this.customers.forEach((g) => { 
								this.results.push({
									distance: this.getDistance(this.x,this.y,g.x,g.y),
									ticket: g.ticket,
									x: g.x,
									y: g.y,
								});
							});

							this.results.sort((a,b) => (a.distance > b.distance) ? 1 : ((b.distance > a.distance) ? -1 : 0));

						},
						download_table_as_csv(table_id, separator = ',') {
							// Select rows from table_id
							var rows = document.querySelectorAll('table#' + table_id + ' tr');
							// Construct csv
							var csv = [];
							for (var i = 0; i < rows.length; i++) {
								var row = [], cols = rows[i].querySelectorAll('td, th');
								for (var j = 0; j < cols.length; j++) {
									// Clean innertext to remove multiple spaces and jumpline (break csv)
									var data = cols[j].innerText.replace(/(\r\n|\n|\r)/gm, '').replace(/(\s\s)/gm, ' ')
									// Escape double-quote with double-double-quote (see https://stackoverflow.com/questions/17808511/properly-escape-a-double-quote-in-csv)
									data = data.replace(/"/g, '""');
									// Push escaped string
									row.push('"' + data + '"');
								}
								csv.push(row.join(separator));
							}
							var csv_string = csv.join('\n');
							// Download it
							var filename = 'export_' + table_id + '_' + new Date().toLocaleDateString() + '__<?php echo $post->post_name;?>.csv';
							var link = document.createElement('a');
							link.style.display = 'none';
							link.setAttribute('target', '_blank');
							link.setAttribute('href', 'data:text/csv;charset=utf-8,' + encodeURIComponent(csv_string));
							link.setAttribute('download', filename);
							document.body.appendChild(link);
							link.click();
							document.body.removeChild(link);
						},
						downloadResults(){
							this.download_table_as_csv('results');
						},
						downloadCustomers(){
							this.download_table_as_csv('results');
						},




					}
				}
			</script> 
		</div>
		<?php echo ob_get_clean();
	}

	function xy_save_postdata( $post_id ) {
		if ( array_key_exists( 'cordx', $_POST ) && array_key_exists( 'cordy', $_POST )) {
			update_post_meta(
				$post_id,
				'_winning_cord_x',
				$_POST['cordx']
			);
			update_post_meta(
				$post_id,
				'_winning_cord_y',
				$_POST['cordy']
			);
		}
	}

}
